from . import BaseActor
import torch.nn.functional as F
import seaborn as sns
import matplotlib.pyplot as plt
import torch

class DiMPActor(BaseActor):
    """Actor for training the DiMP network."""
    def __init__(self, net, objective, loss_weight=None):
        super().__init__(net, objective)
        if loss_weight is None:
            loss_weight = {'iou': 1.0, 'test_clf': 1.0}
        self.loss_weight = loss_weight
    """
    data = TensorDict(11):
      train_images = [3, 10, 3, 288, 288]
      train_anno = [3, 10, 4]
      train_label = [3, 10, 19, 19]
      
      test_images = [3, 10, 3, 288, 288]
      test_anno = [3, 10, 4]
      test_label = [3, 10, 19, 19]
      
      test_proposals = [3, 10, 8, 4]
      proposal_iou = [3, 10, 8]
      
      epoch
      settings
      dataset
    """
    def __call__(self, data):
        """
        args:
            data - The input data, should contain the fields 'train_images', 'test_images', 'train_anno',
                    'test_proposals', 'proposal_iou' and 'test_label'.

        returns:
            loss    - the training loss
            stats  -  dict containing detailed losses
        """
        # Run network target_scores1 是预训练的模型的结果
        target_scores1,target_scores2, iou_pred = self.net(train_imgs=data['train_images'],
                                           test_imgs=data['test_images'],
                                           train_bb=data['train_anno'],
                                           test_proposals=data['test_proposals'])
        # Classification losses for the different optimization iterations
        """
        target_scores1:[list6]: torch.Size([3, 10, 19, 19])
        target_scores2 :torch.Size([3, 10, 37, 37])
        test label :torch.Size([3, 10, 19, 19])
        test anno :torch.Size([3, 10, 4])
        F.interpolate(data['test_label'], size=[37, 37], mode="bilinear")
        """
        label_clone = data['test_label'].clone()
        #clf_losses_test1 = [self.objective['test_clf'](s, label_clone, data['test_anno']) for s in target_scores1]
        clf_losses_test2 = [self.objective['test_clf'](s, data['test_label'], data['test_anno']) for s in target_scores2]

        """
        data['test_label'] = [3, 10, 19, 19]
        data['test_anno'] = [3, 10, 4]
        
        self.objective['iou'] = MSELoss
        self.objective['test_clf'] = LBHinge
        clf_losses_test1 = list[6] : 6 个tensor 1 
        """
        #self.loss_weight ={'iou': 1, 'test_clf': 100, 'test_init_clf': 100, 'test_iter_clf': 400} #100 100 400

        # loss_weight = {'iou': 1, 'test_clf': 100, 'test_init_clf': 100, 'test_iter_clf': 400}
        # Loss of the final filter 第6 次迭代 出来的 filter
       # clf_loss_test1 = clf_losses_test1[-1]
      #  loss_target_classifier1 = self.loss_weight['test_clf1'] * clf_loss_test1

        clf_loss_test2 = clf_losses_test2[-1]
        loss_target_classifier2 = self.loss_weight['test_clf2'] * clf_loss_test2

        # Compute loss for ATOM IoUNet
        loss_iou = self.loss_weight['iou'] * self.objective['iou'](iou_pred, data['proposal_iou'])

        # Loss for the initial filter iteration
        #loss_test_init_clf1 = 0
        loss_test_init_clf2 = 0
        if 'test_init_clf1' in self.loss_weight.keys():
        #    loss_test_init_clf1 = self.loss_weight['test_init_clf1'] * clf_losses_test1[0]
            loss_test_init_clf2 = self.loss_weight['test_init_clf2'] * clf_losses_test2[0]

        # Loss for the intermediate filter iterations
      #  loss_test_iter_clf1 = 0
        loss_test_iter_clf2 = 0
        if 'test_iter_clf1' in self.loss_weight.keys():
            test_iter_weights1 = self.loss_weight['test_iter_clf1']
            test_iter_weights2 = self.loss_weight['test_iter_clf2']
            if isinstance(test_iter_weights1, list):
         #       loss_test_iter_clf1 = sum([a*b for a, b in zip(test_iter_weights1, clf_losses_test1[1:-1])]) #去掉一头一尾 取中间的
                loss_test_iter_clf2 = sum([a*b for a, b in zip(test_iter_weights2, clf_losses_test2[1:-1])])

            else:
          #      loss_test_iter_clf1 = (test_iter_weights1 / (len(clf_losses_test1) - 2)) * sum(clf_losses_test1[1:-1])
                loss_test_iter_clf2 = (test_iter_weights2 / (len(clf_losses_test2) - 2)) * sum(clf_losses_test2[1:-1])

        # Total loss
          #loss_target_classifier1 + loss_test_init_clf1 + loss_test_iter_clf1\
        loss = loss_iou + loss_target_classifier2 + loss_test_init_clf2 + loss_test_iter_clf2

        # Log stats
        stats = {'total_loss': loss.item(),
                 'IOU_loss': loss_iou.item(),

                # '1/loss_final': loss_target_classifier1.item(),
               #  '1/loss_init': loss_test_init_clf1.item(),
              #   '1/loss_iter': loss_test_iter_clf1.item(),

                 '2/loss_final': loss_target_classifier2.item(),
                 '2/loss_init': loss_test_init_clf2.item(),
                 '2/loss_iter': loss_test_iter_clf2.item(),

            #     '1/final': clf_losses_test1[-1].item(),
           #      '1/init': clf_losses_test1[0].item(),
           #      '1/iter': sum(clf_losses_test1[1:-1]).item() / (len(clf_losses_test1) - 2),

                 '2/final': clf_losses_test2[-1].item(),
                 '2/init': clf_losses_test2[0].item(),
                 '2/iter': sum(clf_losses_test2[1:-1]).item() / (len(clf_losses_test2) - 2),
                 }


        return loss, stats
