#-*-coding:utf-8-*-
# import glob
import os
import pandas as pd
import argparse
import numpy as np
import cv2
import time
import sys
sys.path.append(os.getcwd())

from fire import Fire
from tqdm import tqdm

from SiamFC.siamfc import SiamFCTracker

from toolkit.got10k.trackers import Tracker
from toolkit.got10k.experiments import ExperimentOTB, ExperimentVOT



class IdentityTracker(Tracker):
    def __init__(self, model_path, gpu_id, test_name):
        super(IdentityTracker, self).__init__(name=test_name)
        self.siam_tracker = SiamFCTracker(model_path, gpu_id)

    def init(self, image, box):

        image = np.array(image).astype(np.uint8)  # image: img (PIL Image):

        self.siam_tracker.init(image, box)

        self.box = box

    def update(self, image):


        image = np.array(image).astype(np.uint8)  # image: img (PIL Image):

        self.box = self.siam_tracker.update(image)

        return self.box


if __name__ == '__main__':

    # setup tracker
    test_dataset = 'otb100'

    otb_path = '/home/jaffe-h/Documents/Dataset/OTB100'

    vot2018_path = '/home/jaffe-h/Documents/Dataset/VOT/vot2018/VOT2018'


    gpu_id = 0
    model_path = '/home/jaffe-h/Documents/github-Project/VOS/pytracking/SiamFC/models/batch_8/'
    result_dir_got10k = '/home/jaffe-h/Documents/github-Project/VOS/pytracking/result_got10k/result'
    report_dir_got10k = '/home/jaffe-h/Documents/github-Project/VOS/pytracking/result_got10k/report'

    if test_dataset =='otb100' :
        # run experiments on GOT-10k (validation subset)
       for index in range(100):
           model_path1 = (model_path + 'siamfc_train_ILSVRC_b8_' + str(index+1) +'.pth')

           test_name = 'siamfc_train_ILSVRC_b8' + '_' +  str(index+1)
           tracker = IdentityTracker(model_path1, gpu_id, test_name)
           experiment = ExperimentOTB(root_dir = otb_path, version=2015, result_dir=result_dir_got10k, report_dir=report_dir_got10k)
           experiment.run(tracker, visualize=False)

           # report performance
           experiment.report([tracker.name])


    elif test_dataset == 'vot2018':

         for index in range(2):
             test_name = 'siamFC_pretrained_color_gray' + '_' + str(index+1)
             tracker = IdentityTracker(model_path, gpu_id, test_name)
             experiment = ExperimentVOT(root_dir= vot2018_path, version=2018, experiments=('supervised'), result_dir=result_dir_got10k, report_dir=report_dir_got10k)
             experiment.run(tracker, visualize=False)

             # report performance
             experiment.report([tracker.name])
