from pytracking.tracker.base import BaseTracker
import torch
import torch.nn.functional as F
import math
import time
from pytracking import dcf, TensorList
from pytracking.features.preprocessing import numpy_to_torch
from pytracking.utils.plotting import show_tensor, plot_graph
from pytracking.features.preprocessing import sample_patch_multiscale, sample_patch_transformed
from pytracking.features import augmentation
import numpy as np
import seaborn as sns
import matplotlib.pyplot as plt
from pytracking.features import augmentation
import ltr.data.processing_utils as prutils
from ltr import load_network
import cv2
import copy
from pytracking.bbox_fit import fit_bbox_to_mask
from videoanalyst.evaluation import vot_benchmark

class DiMP(BaseTracker):
    def get_axis_aligned_bbox(self, region):
        nv = region.size
        if nv == 8:
            cx = np.mean(region[0::2])
            cy = np.mean(region[1::2])
            x1 = min(region[0::2])
            x2 = max(region[0::2])
            y1 = min(region[1::2])
            y2 = max(region[1::2])
            A1 = np.linalg.norm(region[0:2] -
                                region[2:4]) * np.linalg.norm(region[2:4] -
                                                              region[4:6])
            A2 = (x2 - x1) * (y2 - y1)
            s = np.sqrt(A1 / A2)
            w = s * (x2 - x1) + 1
            h = s * (y2 - y1) + 1

            x= cx - (w)/2
            y= cy - (h)/2

            state = np.array([x, y, w, h])

        else:
            x = region[0]
            y = region[1]
            w = region[2]
            h = region[3]
            cx = x + (w - 1) / 2
            cy = y + (h - 1) / 2
            state = np.array([x, y, w, h])

            self.gt_poly = np.array([state[0], state[1],
                                     state[0] + state[2] - 1, state[1],
                                     state[0] + state[2] - 1, state[1] + state[3] - 1,
                                     state[0], state[1] + state[3] - 1])
            self.rotated_bbox = False
        return state

    def create_dist(self, width, height, cx=None, cy=None):

        if cx is None:
            cx = width / 2
        if cy is None:
            cy = height / 2

        x_ = np.linspace(1, width, width) - cx
        y_ = np.linspace(1, width, width) - cy
        X, Y = np.meshgrid(x_, y_)

        return np.sqrt(np.square(X) + np.square(Y)).astype(np.float32)

    def poly_to_aabbox(self, x_, y_):
        # keep the center and area of the polygon
        # change aspect ratio of the original bbox
        cx = np.mean(x_)
        cy = np.mean(y_)
        x1 = np.min(x_)
        x2 = np.max(x_)
        y1 = np.min(y_)
        y2 = np.max(y_)
        A1 = np.linalg.norm(np.array([x_[0], y_[0]]) - np.array([x_[1], y_[1]])) * \
             np.linalg.norm(np.array([x_[1], y_[1]]) - np.array([x_[2], y_[2]]))
        A2 = (x2 - x1) * (y2 - y1)
        s = np.sqrt(A1 / A2)
        w = s * (x2 - x1) + 1
        h = s * (y2 - y1) + 1
        return np.array([cx - w / 2, cy - h / 2, w, h])

    def init_segmentation(self, image, bb, init_mask=None):
        # bb = np.array[4]
        self.uncertainty_segm_scale_thr = 3.5
        self.segm_pixels_ratio = 2
        self.mask_pixels_budget_sz = 25
        self.search_area_scale = 4.5
        self.segm_min_scale = 0.2
        self.max_rel_scale_ch_thr = 0.75
        self.max_scale_change_factor = 1.05
        self.min_scale_change_factor = 0.95
        self.consider_segm_pixels_ratio = 1

        self.segm_scale_estimation = True
        self.segmentation_task = False

        init_patch_crop, f_ = prutils.sample_target(image, np.array(bb), 4,
                                                    output_sz=24*16)
        """
        init_patch_crop = np.array[384, 384, 3], f_= 1.5609
        """

        """
        mask = np.array[352, 624] 以boundingbox 为mask
        """
        mask = np.zeros((image.shape[0], image.shape[1]), dtype=np.int32)

        if hasattr(self, 'gt_poly'):
            p1 = self.gt_poly[:2]
            p2 = self.gt_poly[2:4]
            p3 = self.gt_poly[4:6]
            p4 = self.gt_poly[6:]
            cv2.fillConvexPoly(mask, np.array([p1, p2, p3, p4], dtype=np.int32), 1)
            mask = mask.astype(np.float32)
        else:
            p1 = bb[:2]
            p2 = [bb[0] + bb[2], bb[1]]
            p3 = [bb[0] + bb[2], bb[1] + bb[3]]
            p4 = [bb[0], bb[1] + bb[3]]
            cv2.fillConvexPoly(mask, np.array([p1, p2, p3, p4], dtype=np.int32), 1)
            mask = mask.astype(np.float32)

        init_mask_patch_np, patch_factor_init = prutils.sample_target(mask, np.array(bb),
                                                                      4.0, output_sz=384, pad_val=0)
        """
        init_mask_patch_np = np.array[384, 384] 周围有渐变
        patch_factor_init = 1.5609
        """
        # network was renamed therefore we need to specify constructor_module and constructor_fun_name
        segment_netpath = '/home/jaffe/PycharmProjects/pytracking-platform1/pytracking/networks/SegmNet.pth.tar'
        segm_net, _ = load_network(segment_netpath, backbone_pretrained=False,
                                   constructor_module='ltr.models.segm.segm',
                                   constructor_fun_name='segm_resnet50')

        segm_net.cuda()
        segm_net.eval()

        for p in segm_net.segm_predictor.parameters():
            p.requires_grad = False

        self.segm_normalize_mean = np.array([0.485, 0.456, 0.406]).reshape((1, 1, 3))
        self.segm_normalize_std = np.array([0.229, 0.224, 0.225]).reshape((1, 1, 3))

        # normalize input image
        init_patch_norm_ = init_patch_crop.astype(np.float32) / float(255)
        init_patch_norm_ -= self.segm_normalize_mean
        init_patch_norm_ /= self.segm_normalize_std

        # create distance map for discriminative segmentation
        # center-based dist map
        dist_map = self.create_dist(init_patch_crop.shape[0], init_patch_crop.shape[1])
        """
        dist_map = np.array[384, 384]
        """
        dist_map = torch.Tensor(dist_map)

        # put image patch and mask to GPU
        init_patch = torch.Tensor(init_patch_norm_)
        init_mask_patch = torch.Tensor(init_mask_patch_np)
        """
        init_patch = tensor[384, 384, 3]
        init_mask_patch = tensor[384, 384]
        """
        init_patch = init_patch.to(self.params.device)
        init_mask_patch = init_mask_patch.to(self.params.device)
        dist_map = dist_map.to(self.params.device)
        dist_map = torch.unsqueeze(torch.unsqueeze(dist_map, dim=0), dim=0)
        test_dist_map = [dist_map]


        # reshape image for the feature extractor
        init_patch = torch.unsqueeze(init_patch, dim=0).permute(0, 3, 1, 2)
        init_mask_patch = torch.unsqueeze(torch.unsqueeze(init_mask_patch, dim=0), dim=0)
        """
         init_patch = tensor[1, 3, 384, 384]
         init_mask_patch = tensor[1, 1, 384, 384]
        """

        # extract features (extracting twice on the same patch - not necessary)
        train_feat = segm_net.extract_backbone_features(init_patch)
        """
        train_feat = oredereddict(4): 
        conv1:[1, 64, 192, 192]
        layer1:[1, 256, 96, 96]
        layer2:[1, 512, 48, 48]
        layer3:[1, 1024, 24, 24]
        """
        # prepare features in the list (format for the network)
        train_feat_segm = [feat for feat in train_feat.values()]
        test_feat_segm = [feat for feat in train_feat.values()]
        train_masks = [init_mask_patch]
        """
        train_feat_segm = list(4): 
        conv1:[1, 64, 192, 192]
        layer1:[1, 256, 96, 96]
        layer2:[1, 512, 48, 48]
        layer3:[1, 1024, 24, 24]

        test_feat_segm = train_feat_segm
        """

        if init_mask is None:
            iters = 0
            while iters < 1:
                # Obtain segmentation prediction
                segm_pred = segm_net.segm_predictor(test_feat_segm, train_feat_segm, train_masks, test_dist_map)
                """
                segm_pred = [1, 2, 384, 384]
                """

                # softmax on the prediction (during training this is done internaly when calculating loss)
                # take only the positive channel as predicted segmentation mask
                mask = F.softmax(segm_pred, dim=1)[0, 0, :, :].cpu().numpy()
                mask = (mask > 0.5).astype(np.float32)
                """
                mask = np.array[384, 384] : 0 or 1
                self.params.init_segm_mask_thr=0.5
                """

                if hasattr(self, 'gt_poly'):
                    # dilate polygon-based mask
                    # dilate only if given mask is made from polygon, not from axis-aligned bb (since rotated bb is much tighter)
                    dil_kernel_sz = max(5, int(round(0.05 * min(self.target_sz).item() * f_)))
                    kernel = np.ones((dil_kernel_sz, dil_kernel_sz), np.uint8)
                    mask_dil = cv2.dilate(init_mask_patch_np, kernel, iterations=1)
                    """
                    膨胀操作
                    """
                    mask = mask * mask_dil
                    """
                    dil_kernel_sz = 5
                    kernel = [5, 5]: 全是1 
                    mask_dil = np.array[384, 384]
                    """
                else:
                    mask = mask * init_mask_patch_np

                target_pixels = np.sum((mask > 0.5).astype(np.float32))
                self.segm_init_target_pixels = target_pixels


                mask_gpu = torch.unsqueeze(torch.unsqueeze(torch.tensor(mask), dim=0), dim=0).to(self.params.device)
                train_masks = [mask_gpu]

                iters += 1

        # store everything that is needed for later
        self.segm_net = segm_net
        self.train_feat_segm = train_feat_segm
        self.init_mask_patch = mask_gpu
        self.dist_map = dist_map
        self.mask_pixels = np.array([np.sum(mask)])  # np.array[1]: 7539.37
        self.segm_optimize_polygon = True

    def segment_target(self, image, pos, sz):
        # new_pos, self.target_sz
        # pos and sz are in the image coordinates
        # construct new bounding box first


        """
        self.img_sample_sz[0]
        self.uncert_score
        self.target_scale

        """
        self.segm_scale_estimation = True
        self.segm_scale_estimation = False
        self.consider_segm_pixels_ratio = 1
        self.segm_pixels_ratio = 2

        """
        self.segm_net = segm_net
        self.train_feat_segm = train_feat_segm
        self.init_mask_patch = mask_gpu
        self.dist_map = dist_map
        self.mask_pixels = np.array([np.sum(mask)])  # np.array[1]: 7539.37
        self.segmentation_task = False
        self.segm_optimize_polygon
        """
        tlx_ = pos[1] - sz[1] / 2
        tly_ = pos[0] - sz[0] / 2
        w_ = sz[1]
        h_ = sz[0]
        bb = [tlx_.item(), tly_.item(), w_.item(), h_.item()]

        # extract patch
        patch, f_ = prutils.sample_target(image, np.array(bb), 4,
                                          output_sz=384)

        segm_crop_sz = math.ceil(math.sqrt(bb[2] * bb[3]) * 4)

        # normalize input image
        init_patch_norm_ = patch.astype(np.float32) / float(255)
        init_patch_norm_ -= self.segm_normalize_mean
        init_patch_norm_ /= self.segm_normalize_std

        # put image patch and mask to GPU
        patch_gpu = torch.Tensor(init_patch_norm_)
        patch_gpu = patch_gpu.to(self.params.device)

        # reshape image for the feature extractor
        patch_gpu = torch.unsqueeze(patch_gpu, dim=0).permute(0, 3, 1, 2)

        # extract features (extracting twice on the same patch - not necessary)
        test_feat = self.segm_net.extract_backbone_features(patch_gpu)

        # prepare features in the list (format for the network)
        test_feat_segm = [feat for feat in test_feat.values()]
        train_masks = [self.init_mask_patch]
        # center-based distance map
        test_dist_map = [self.dist_map]



        # Obtain segmentation prediction
        segm_pred = self.segm_net.segm_predictor(test_feat_segm, self.train_feat_segm, train_masks, test_dist_map)

        # softmax on the prediction (during training this is done internaly when calculating loss)
        # take only the positive channel as predicted segmentation mask
        mask = F.softmax(segm_pred, dim=1)[0, 0, :, :].cpu().numpy()

        mask = (mask > 0.5).astype(np.uint8)

        if cv2.__version__[-5] == '4':
            contours, _ = cv2.findContours(mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_NONE)
        else:
            _, contours, _ = cv2.findContours(mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_NONE)
        cnt_area = [cv2.contourArea(cnt) for cnt in contours]

        if len(cnt_area) > 0 and len(contours) != 0 and np.max(cnt_area) > 50:  # 1000:
            contour = contours[np.argmax(cnt_area)]  # use max area polygon
            polygon = contour.reshape(-1, 2)

            prbox = np.reshape(cv2.boxPoints(cv2.minAreaRect(polygon)), (4, 2))  # Rotated Rectangle
            prbox_init = copy.deepcopy(prbox)

            prbox_opt = np.array([])
            if self.segm_optimize_polygon:
                if not self.segmentation_task:
                    mask = np.zeros(mask.shape, dtype=np.uint8)
                    cv2.drawContours(mask, [contour], -1, 1, thickness=-1)
                    # save mask to disk
                    # Note: move this below if evaluating on VOT


                t_opt_start_ = time.time()
                prbox_opt_ = fit_bbox_to_mask(mask.astype(np.int32), rotated=self.rotated_bbox)
                """
                prbox_opt_ = np.array(4, 2)
                """
              #  mask_show = mask.astype(np.int32)
            #    sns.heatmap(mask_show)
            #    plt.show()
                bbox_opt_time = time.time() - t_opt_start_
                if prbox_opt_ is not None:
                    A1 = np.linalg.norm(np.array([prbox[0, 0], prbox[0, 1]]) - np.array([prbox[1, 0], prbox[1, 1]])) * \
                         np.linalg.norm(np.array([prbox[1, 0], prbox[1, 1]]) - np.array([prbox[2, 0], prbox[2, 1]]))
                    A_new = np.linalg.norm(np.array([prbox_opt_[0, 0], prbox_opt_[0, 1]]) - np.array(
                        [prbox_opt_[1, 0], prbox_opt_[1, 1]])) * \
                            np.linalg.norm(np.array([prbox_opt_[1, 0], prbox_opt_[1, 1]]) - np.array(
                                [prbox_opt_[2, 0], prbox_opt_[2, 1]]))
                    area_ratio = A_new / A1

                    if area_ratio > 0.1 and area_ratio < 2.5:  # 1.7
                        prbox_opt = prbox_opt_
                    else:
                        print('Bbox optimization has made too large difference.')

            displacement = np.mean(prbox, axis=0) - np.array([mask.shape[0] / 2, mask.shape[1] / 2])
            prbox = (prbox - np.mean(prbox, axis=0) + displacement) / f_ + np.array([pos[1].item(), pos[0].item()])


            if self.segm_scale_estimation:
                # use pixels_ratio to determine if new scale should be estimated or not
                mask_pixels_ = np.max(cnt_area)
                pixels_ratio = abs(np.mean(self.mask_pixels) - mask_pixels_) / np.mean(self.mask_pixels)
                if self.uncert_score < self.uncertainty_segm_scale_thr:
                    if pixels_ratio < self.segm_pixels_ratio:  # 0.6:
                        self.mask_pixels = np.append(self.mask_pixels, mask_pixels_)
                        if self.mask_pixels.size > self.mask_pixels_budget_sz:
                            self.mask_pixels = np.delete(self.mask_pixels, 0)

                        new_aabb = self.poly_to_aabbox(prbox[:, 0], prbox[:, 1])
                        new_target_scale = (math.sqrt(new_aabb[2] * new_aabb[3]) * 4.5) / \
                                           self.img_sample_sz_seg[0]
                        rel_scale_ch = (abs(new_target_scale - self.target_scale_seg) / self.target_scale_seg).item()
                        if new_target_scale > self.segm_min_scale and rel_scale_ch < self.max_rel_scale_ch_thr:
                            self.target_scale = max(self.target_scale * self.min_scale_change_factor,
                                                    min(self.target_scale * self.max_scale_change_factor,
                                                        new_target_scale))

            seg_posCY = np.mean(prbox[:, 1])
            seg_posCX = np.mean(prbox[:, 0])
            delta_x = abs(seg_posCX - pos.cpu().numpy()[1])
            delta_y = abs(seg_posCY - pos.cpu().numpy()[0])
            dis_seg = math.sqrt((pow(delta_x, 2) + pow(delta_y, 2)))


            if not self.segm_scale_estimation or pixels_ratio < self.consider_segm_pixels_ratio:
                self.pos[0] = np.mean(prbox[:, 1])
                self.pos[1] = np.mean(prbox[:, 0])
                #pass

            if not self.segm_scale_estimation or pixels_ratio < self.segm_pixels_ratio:
                if prbox_opt.size > 0:
                    displacement_opt = np.mean(prbox_opt, axis=0) - np.array([mask.shape[0] / 2, mask.shape[1] / 2])
                    prbox = (prbox_opt - np.mean(prbox_opt, axis=0) + displacement_opt) / f_ + np.array(
                        [pos[1].item(), pos[0].item()])

                if self.rotated_bbox:
                    pred_region = [prbox[0, 0], prbox[0, 1], prbox[1, 0], prbox[1, 1], prbox[2, 0], prbox[2, 1],
                                   prbox[3, 0], prbox[3, 1]]
                else:
                    pred_region = [np.min(prbox[:, 0]) + 1, np.min(prbox[:, 1]) + 1,
                                   np.max(prbox[:, 0]) - np.min(prbox[:, 0]) + 1,
                                   np.max(prbox[:, 1]) - np.min(prbox[:, 1]) + 1]
                a= math.sqrt(self.image.shape[0]*self.image.shape[1])
              #  if dis_seg > (a/40):
                    # self.over_dispacement = self.over_dispacement + 1
              #      print('over displacement')
              #      return None


                #print(str(b_overlap))


                self.seg_pos_y = np.mean(prbox[:, 1])
                self.seg_pos_x = np.mean(prbox[:, 0])
                pos_x = self.pos[1].item()
                pos_y = self.pos[0].item()
                delat_x = self.seg_pos_x - pos_x
                delat_y = self.seg_pos_y - pos_y

                delat_x_relative = delat_x*0.5
                delat_y_relative = delat_y*0.5
                pred_region_delta = [prbox[0, 0] - delat_x_relative, prbox[0, 1] - delat_y_relative,
                                     prbox[1, 0] - delat_x_relative, prbox[1, 1] - delat_y_relative,
                                     prbox[2, 0] - delat_x_relative, prbox[2, 1] - delat_y_relative,
                                     prbox[3, 0] - delat_x_relative, prbox[3, 1] - delat_y_relative]
                b_overlap = vot_benchmark.vot_overlap(
                    self.new_state_list, pred_region_delta, (self.image.shape[1], self.image.shape[0]))
                if b_overlap < 0.35:
                 #   print('warning less area')
                    return None

                #print("deta_x:" + str(delat_x) +", delta_y: "+ str(delat_y))
                cal =np.array(pred_region)
                post_x = np.mean(cal[0:8:2])
                post_y = np.mean(cal[1:8:2])
               # if (post_y- pos_y) > 0.2 or (post_x- pos_x) > 0.2:
               #     print('center not match')
                # print('seg') pred_region
                pred_np =np.array(pred_region_delta)
                self.pos[0] = np.mean(pred_np[1:8:2])
                self.pos[1] = np.mean(pred_np[0:8:2])
                return pred_region_delta

        return None

    def use_iouNet(self, useiou=1):
        self.use_iouNet_flag = useiou

    def initialize_features(self):
        if not getattr(self, 'features_initialized', False):
            self.params.net.initialize()
        self.features_initialized = True

    def initialize(self, image, info, seg_flag = True) -> dict:
        if isinstance(info, dict):
            state = info['init_bbox']
            state = np.array(state)
        else:
            state = info
        self.seg_flag = seg_flag
        #print('Use segmentaion or not: '+str(self.seg_flag))
        state = self.get_axis_aligned_bbox(state)
        # Initialize some stuff
        self.frame_num = 1
        if not hasattr(self.params, 'device'):
            self.params.device = 'cuda' if self.params.use_gpu else 'cpu'

        # Initialize network
        self.initialize_features()

        # The DiMP network
        """
        pytracking.features.net_wrappers.NetWithBackbone {net = DiMPnet, net_path, use_gpu}

        """
        self.net = self.params.net

        # Time initialization
        tic = time.time()

        # Get target position and size
        self.pos = torch.Tensor([state[1] + (state[3] - 1) / 2, state[0] + (state[2] - 1) / 2])
        self.target_sz = torch.Tensor([state[3], state[2]])

        # Set sizes
        sz = self.params.image_sample_size
        self.img_sample_sz = torch.Tensor([sz, sz] if isinstance(sz, int) else sz)
        self.img_support_sz = self.img_sample_sz

        # Set search area
        search_area = torch.prod(self.target_sz * self.params.search_area_scale).item()
        self.target_scale = math.sqrt(search_area) / self.img_sample_sz.prod().sqrt()

        # Target size in base scale
        self.base_target_sz = self.target_sz / self.target_scale

        # Convert image
        im = numpy_to_torch(image)

        # Setup scale factors
        if not hasattr(self.params, 'scale_factors'):
            self.params.scale_factors = torch.ones(1)
        elif isinstance(self.params.scale_factors, (list, tuple)):
            self.params.scale_factors = torch.Tensor(self.params.scale_factors)

        # Setup scale bounds
        self.image_sz = torch.Tensor([im.shape[2], im.shape[3]])
        self.min_scale_factor = torch.max(10 / self.base_target_sz)
        self.max_scale_factor = torch.min(self.image_sz / self.base_target_sz)

        # Extract and transform sample
        """
        init_backbone_feat = orderdict{
        layer2 = tensor [23, 512, 28, 28]
        layer3 = tensor [23, 512, 14, 14] }
        """
        init_backbone_feat_iou, init_backbone_feat_secondFilter = self.generate_init_samples(im)

        # Initialize classifier
        """
         初始化 滤波器！！！！
        """
        self.init_classifier(init_backbone_feat_iou, init_backbone_feat_secondFilter)

        #####################################################################################################
        self.pos_seg = self.pos.clone()
        self.target_sz_seg  = torch.Tensor([state[3], state[2]])
        self.target_scale_seg = 1.0
        search_area_seg = torch.prod(self.target_sz_seg * 4.5).item()
        if search_area_seg > (256*256):
            self.target_scale_seg = math.sqrt(search_area_seg / 256)
        elif search_area_seg < (256*256):
            self.target_scale_seg = math.sqrt(search_area_seg / 256)

        # Target size in base scale
        self.base_target_sz_seg = self.target_sz_seg / self.target_scale_seg

        # Use odd square search area and set sizes
        feat_max_stride_seg = 16
        self.img_sample_sz_seg = torch.round(
                torch.sqrt(torch.prod(self.base_target_sz_seg * 4.5))) * torch.ones(2)

        self.img_support_sz_seg = self.img_sample_sz_seg
        self.output_sz_seg = 1 * self.img_support_sz_seg  # Interpolated size of the output
        self.scores_seg = np.array([1])
        self.frame_num = 1
        self.tracking_uncertainty_thr = 3.0
        self.response_budget_sz = 25
        self.tracking_uncertainty_thr = 3
        self.scores = np.array([1])

        ######################################################################################################
        if seg_flag == True:
           self.init_segmentation(image, state, init_mask=None)

        # Initialize IoUNet
        if getattr(self.params, 'use_iou_net', True):
            self.init_iou_net(init_backbone_feat_iou)

        out = {'time': time.time() - tic}
        return out

    def init(self, image, gt):
        self.rotated_bbox = True
        self.over_dispacement = 0

        out = self.initialize(image, gt)
        return out

    def track(self, image, info: dict = None) -> dict:
        self.debug_info = {}

        self.frame_num += 1
        self.debug_info['frame_num'] = self.frame_num

        # Convert image
        im = numpy_to_torch(image)
        self.image = image

        # ------- LOCALIZATION ------- #

        # Extract backbone features
        """
        backbone_feat = orderdict{
        layer2 = tensor [23, 512, 28, 28]
        layer3 = tensor [23, 512, 14, 14] }
        sample_coords = tensor[1, 4]
        """
        iou_first_backbone_feature, secondF_backbone_feature, sample_coords = self.extract_backbone_features(im, self.get_centered_sample_pos(),
                                                                      self.target_scale * self.params.scale_factors,
                                                                      self.img_sample_sz)  # img_sample_sz = [224, 224]
        # Extract classification features
        """
        test_x = [1, 512, 14, 14]

        test_x2 = [1, 256, 28, 28]
        test_x1 = [1, 512, 14, 14]
        """
        # test_x1 是 secondF
        test_x1, test_x2 = self.get_classification_features(iou_first_backbone_feature, secondF_backbone_feature)

        # Location of sample
        """
        sample_pos = [1, 2 ] cx, cy
        """
        sample_pos, sample_scales = self.get_sample_location(sample_coords)

        # Compute classification scores
        """
        scores_raw = tensor[1, 1, 15, 15] 
        layer2: [1, 1, 29, 29]
        """
        # scores_raw2 是pretrained 的分数
        scores = self.classify_target(test_x1, test_x2)

        ###########################################################################################
        max_score = torch.max(scores).item()
        uncert_score = 0

        #print(str(max_score))
        if self.frame_num > 5:
            uncert_score = np.mean(self.scores) / max_score
        self.uncert_score = uncert_score

        self.tracking_uncertainty_thr = 3
        if uncert_score < self.tracking_uncertainty_thr: # tracking_uncertainty_thr = 3
            self.scores = np.append(self.scores, max_score)
            if self.scores.size > self.response_budget_sz: # response_budget_sz = 25
                self.scores = np.delete(self.scores, 0) # 移除最早的分数

        ###########################################################################################

        # Localize the target
        translation_vec, scale_ind, s, flag = self.localize_target(scores, sample_scales)
        new_pos = sample_pos[scale_ind, :] + translation_vec
        """
        translation_vec = tensor[2]
        scale_ind = tensor
        s = 
        """

        self.debug_info['flag'] = flag

        # Update position and scale
        if flag != 'not_found':
            if getattr(self.params, 'use_iou_net', True):
                update_scale_flag = getattr(self.params, 'update_scale_when_uncertain', True) or flag != 'uncertain'
                if getattr(self.params, 'use_classifier', True):
                    self.update_state(new_pos)

                self.refine_target_box(iou_first_backbone_feature, sample_pos[scale_ind, :], sample_scales[scale_ind], scale_ind,
                                       update_scale_flag)
            elif getattr(self.params, 'use_classifier', True):
                self.update_state(new_pos, sample_scales[scale_ind])

        # ------- UPDATE ------- #

        update_flag = flag not in ['not_found', 'uncertain']
        hard_negative = (flag == 'hard_negative')
        learning_rate = getattr(self.params, 'hard_negative_learning_rate', None) if hard_negative else None

        if getattr(self.params, 'update_classifier', False) and update_flag:
            # Get train sample
            train_x1 = test_x1[scale_ind:scale_ind + 1, ...]
            train_x2 = test_x2[scale_ind:scale_ind+1, ...]

            # Create target_box and label for spatial sample
            target_box = self.get_iounet_box(self.pos, self.target_sz, sample_pos[scale_ind, :],
                                             sample_scales[scale_ind])
           # target_box[2:4:1] =target_box[2:4:1] * 0.9
            # Update the classifier model
            self.update_classifier(train_x1, train_x2, target_box, learning_rate, s[scale_ind, ...])  # train_x2

        # Set the pos of the tracker to iounet pos
        if getattr(self.params, 'use_iou_net', True) and flag != 'not_found' and hasattr(self, 'pos_iounet'):
            self.pos = self.pos_iounet.clone()

        score_map = s[scale_ind, ...]
        max_score = torch.max(score_map).item()
        self.debug_info['max_score'] = max_score

        if self.visdom is not None:
            self.visdom.register(score_map, 'heatmap', 2, 'Score Map')
            self.visdom.register(self.debug_info, 'info_dict', 1, 'Status')
        elif self.params.debug >= 2:
            show_tensor(score_map, 5, title='Max score = {:.2f}'.format(max_score))

        # Compute output bounding box
        new_state = torch.cat((self.pos[[1, 0]] - (self.target_sz[[1, 0]] - 1) / 2, self.target_sz[[1, 0]]))
        new_state =new_state.tolist()
        new_state_8points = (new_state[0], new_state[1],
                     new_state[0] + new_state[2], new_state[1],
                     new_state[0] + new_state[2],
                     new_state[1] + new_state[3], new_state[0],
                     new_state[1] + new_state[3])

        self.new_state_list = new_state_8points
        """
        im, self.get_centered_sample_pos(),
        self.target_scale * self.params.scale_factors,
        self.img_sample_sz
        """
        ########################################################################
        if flag == 'not_found':
            uncert_score = 100

        image_seg = image.copy()
        pos_seg = self.pos.clone()
        target_sz_seg = self.target_sz.clone()
        a = self.pos.clone()
        b = self.target_sz.clone()
        ts = self.target_scale.clone()
        sz = self.img_sample_sz.clone()

        self.uncertainty_segment_thr = 2.1
       # print(str(uncert_score))
        if flag not in ['not_found', 'uncertain'] and uncert_score < self.uncertainty_segment_thr: # 10
            if self.seg_flag == True:
               seg_out = self.segment_target(image_seg, pos_seg, target_sz_seg )
            else:
               seg_out = None
        else:
            if self.seg_flag == True:
          #     print('not seg, uncer = '+ str(uncert_score))
               pass
            seg_out = None

       # c = (a[0] - self.pos[0].clone()).cpu().numpy()
       # d = (b[0] - self.target_sz[0].clone()).cpu().numpy()
      #  e = (ts - self.target_scale.clone()).cpu().numpy()
      #  f = (sz[0] - self.img_sample_sz[0].clone()).cpu().numpy()

        #if c != 0.0 or d !=0.0 or e !=0.0 or f !=0.0:
        #   print('pos' + str(c))
        #   print('target' + str(d))
        ############################################################################

       # out = {'target_bbox': new_state}  new_state_8points
        out = {'target_bbox': new_state_8points,
               'seg_out':seg_out,
               'no_seg_4p': new_state}
        return out

    def get_Cxy_pos_np(self, ):
        pos_tensor = self.pos.clone()
        pos = pos_tensor.cpu().numpy()
        return pos[[1, 0]]

    def update_state_pos_sz(self, xywh):
        pos = torch.from_numpy(xywh[0:2])
        wh = torch.from_numpy(xywh[2:4])
        self.pos = pos[[1, 0]]
        self.target_sz = wh[[1, 0]]

    def update(self, image):
        out = self.track(image)
        if  out['seg_out'] == None:
            final_out = out['target_bbox']
        else:
            final_out = out['seg_out']


        return final_out

    def update_got10K_4p(self, image):
        out = self.track(image)

        final_out = out['no_seg_4p']


        return final_out


    def get_sample_location(self, sample_coord):
        """Get the location of the extracted sample."""
        sample_coord = sample_coord.float()
        sample_pos = 0.5 * (sample_coord[:, :2] + sample_coord[:, 2:] - 1)
        sample_scales = ((sample_coord[:, 2:] - sample_coord[:, :2]) / self.img_sample_sz).prod(dim=1).sqrt()
        return sample_pos, sample_scales

    def get_centered_sample_pos(self):
        """Get the center position for the new sample. Make sure the target is correctly centered."""
        a =self.pos +  ((self.feature_sz + self.kernel_size) % 2) * self.target_scale * \
            self.img_support_sz / (2 * self.feature_sz)
        return a

    def classify_target(self, sample_x1: TensorList, sample_x2: TensorList):  # sample_x2: TensorList
        """Classify target by applying the DiMP filter."""
        with torch.no_grad():
            scores_raw_pretrained = self.net.dimp_pretrained.classifier.classify(self.target_filter2, sample_x2)
            scores_raw_scecondF = self.net.classifier.classify(self.target_filter1, sample_x1)
            score = 0.2* scores_raw_pretrained + 0.8* scores_raw_scecondF
        return score

    def localize_target(self, scores, sample_scales):
        """Run the target localization."""

        scores = scores.squeeze(1)

        if getattr(self.params, 'advanced_localization', False):
            return self.localize_advanced(scores, sample_scales)

        # Get maximum
        score_sz = torch.Tensor(list(scores.shape[-2:]))
        score_center = (score_sz - 1) / 2
        max_score, max_disp = dcf.max2d(scores)
        _, scale_ind = torch.max(max_score, dim=0)
        max_disp = max_disp[scale_ind, ...].float().cpu().view(-1)
        target_disp = max_disp - score_center

        # Compute translation vector and scale change factor
        translation_vec = target_disp * (self.img_support_sz / self.feature_sz) * sample_scales[scale_ind]

        return translation_vec, scale_ind, scores, None

    def localize_advanced(self, scores, sample_scales):
        """Run the target advanced localization (as in ATOM)."""

        sz = scores.shape[-2:]
        score_sz = torch.Tensor(list(sz))
        score_center = (score_sz - 1) / 2

        scores_hn = scores
        if self.output_window is not None and getattr(self.params, 'perform_hn_without_windowing', False):
            scores_hn = scores.clone()
            scores *= self.output_window

        max_score1, max_disp1 = dcf.max2d(scores) # T[1, 15, 15]
        _, scale_ind = torch.max(max_score1, dim=0)
        sample_scale = sample_scales[scale_ind]
        max_score1 = max_score1[scale_ind]
        max_disp1 = max_disp1[scale_ind, ...].float().cpu().view(-1)
        target_disp1 = max_disp1 - score_center
        translation_vec1 = target_disp1 * (self.img_support_sz / self.feature_sz) * sample_scale

        if max_score1.item() < self.params.target_not_found_threshold:
            return translation_vec1, scale_ind, scores_hn, 'not_found'

        # Mask out target neighborhood
        target_neigh_sz = self.params.target_neighborhood_scale * (self.target_sz / sample_scale) * (
                    self.feature_sz / self.img_support_sz)

        tneigh_top = max(round(max_disp1[0].item() - target_neigh_sz[0].item() / 2), 0)
        tneigh_bottom = min(round(max_disp1[0].item() + target_neigh_sz[0].item() / 2 + 1), sz[0])
        tneigh_left = max(round(max_disp1[1].item() - target_neigh_sz[1].item() / 2), 0)
        tneigh_right = min(round(max_disp1[1].item() + target_neigh_sz[1].item() / 2 + 1), sz[1])
        scores_masked = scores_hn[scale_ind:scale_ind + 1, ...].clone()
        scores_masked[..., tneigh_top:tneigh_bottom, tneigh_left:tneigh_right] = 0

        # Find new maximum
        max_score2, max_disp2 = dcf.max2d(scores_masked)
        max_disp2 = max_disp2.float().cpu().view(-1)
        target_disp2 = max_disp2 - score_center
        translation_vec2 = target_disp2 * (self.img_support_sz / self.feature_sz) * sample_scale

        # Handle the different cases
        if max_score2 > self.params.distractor_threshold * max_score1:
            disp_norm1 = torch.sqrt(torch.sum(target_disp1 ** 2))
            disp_norm2 = torch.sqrt(torch.sum(target_disp2 ** 2))
            disp_threshold = self.params.dispalcement_scale * math.sqrt(sz[0] * sz[1]) / 2

            if disp_norm2 > disp_threshold and disp_norm1 < disp_threshold:
                return translation_vec1, scale_ind, scores_hn, 'hard_negative'
            if disp_norm2 < disp_threshold and disp_norm1 > disp_threshold:
                return translation_vec2, scale_ind, scores_hn, 'hard_negative'
            if disp_norm2 > disp_threshold and disp_norm1 > disp_threshold:
                return translation_vec1, scale_ind, scores_hn, 'uncertain'

            # If also the distractor is close, return with highest score
            return translation_vec1, scale_ind, scores_hn, 'uncertain'

        if max_score2 > self.params.hard_negative_threshold * max_score1 and max_score2 > self.params.target_not_found_threshold:
            return translation_vec1, scale_ind, scores_hn, 'hard_negative'

        return translation_vec1, scale_ind, scores_hn, 'normal'

    def extract_backbone_features(self, im: torch.Tensor, pos: torch.Tensor, scales, sz: torch.Tensor):
        im_patches, patch_coords = sample_patch_multiscale(im, pos, scales, sz,
                                                           getattr(self.params, 'border_mode', 'replicate'))
        """
        im_patches = [1, 3, 224, 224]
        patch_coords = [42, 398, 177, 533]
        """
        with torch.no_grad():
            iou_first_backbone_feature, secondF_backbone_feature = self.net.extract_backbone(im_patches)
        return iou_first_backbone_feature, secondF_backbone_feature, patch_coords

    def get_classification_features(self, init_backbone_feat_iou, init_backbone_feat_secondF):
        with torch.no_grad():
            return self.net.extract_classification_feat(init_backbone_feat_iou, init_backbone_feat_secondF)

    def get_iou_backbone_features(self, backbone_feat):
        return self.net.get_backbone_bbreg_feat(backbone_feat)

    def get_iou_features(self, backbone_feat):
        with torch.no_grad():
            return self.net.dimp_pretrained.bb_regressor.get_iou_feat(self.get_iou_backbone_features(backbone_feat))

    def get_iou_modulation(self, iou_backbone_feat, target_boxes):
        with torch.no_grad():
            return self.net.dimp_pretrained.bb_regressor.get_modulation(iou_backbone_feat, target_boxes)

    def generate_init_samples(self, im: torch.Tensor) -> TensorList:
        """Perform data augmentation to generate initial training samples."""

        if getattr(self.params, 'border_mode', 'replicate') == 'inside':
            # Get new sample size if forced inside the image
            im_sz = torch.Tensor([im.shape[2], im.shape[3]])
            sample_sz = self.target_scale * self.img_sample_sz
            shrink_factor = (sample_sz.float() / im_sz).max().clamp(1)
            sample_sz = (sample_sz.float() / shrink_factor)
            self.init_sample_scale = (sample_sz / self.img_sample_sz).prod().sqrt()
            tl = self.pos - (sample_sz - 1) / 2
            br = self.pos + sample_sz / 2 + 1
            global_shift = - ((-tl).clamp(0) - (br - im_sz).clamp(0)) / self.init_sample_scale
        else:
            self.init_sample_scale = self.target_scale
            global_shift = torch.zeros(2)

        self.init_sample_pos = self.pos.round()

        # Compute augmentation size
        aug_expansion_factor = getattr(self.params, 'augmentation_expansion_factor', None)
        aug_expansion_sz = self.img_sample_sz.clone()
        aug_output_sz = None
        if aug_expansion_factor is not None and aug_expansion_factor != 1:
            aug_expansion_sz = (self.img_sample_sz * aug_expansion_factor).long()
            aug_expansion_sz += (aug_expansion_sz - self.img_sample_sz.long()) % 2
            aug_expansion_sz = aug_expansion_sz.float()
            aug_output_sz = self.img_sample_sz.long().tolist()

        # Random shift for each sample
        get_rand_shift = lambda: None
        random_shift_factor = getattr(self.params, 'random_shift_factor', 0)
        if random_shift_factor > 0:
            get_rand_shift = lambda: (
                        (torch.rand(2) - 0.5) * self.img_sample_sz * random_shift_factor + global_shift).long().tolist()

        # Always put identity transformation first, since it is the unaugmented sample that is always used
        self.transforms = [augmentation.Identity(aug_output_sz, global_shift.long().tolist())]

        augs = self.params.augmentation if getattr(self.params, 'use_augmentation', True) else {}

        # Add all augmentations
        if 'shift' in augs:
            self.transforms.extend(
                [augmentation.Translation(shift, aug_output_sz, global_shift.long().tolist()) for shift in
                 augs['shift']])
        if 'relativeshift' in augs:
            get_absolute = lambda shift: (torch.Tensor(shift) * self.img_sample_sz / 2).long().tolist()
            self.transforms.extend(
                [augmentation.Translation(get_absolute(shift), aug_output_sz, global_shift.long().tolist()) for shift in
                 augs['relativeshift']])
        if 'fliplr' in augs and augs['fliplr']:
            self.transforms.append(augmentation.FlipHorizontal(aug_output_sz, get_rand_shift()))
        if 'blur' in augs:
            self.transforms.extend(
                [augmentation.Blur(sigma, aug_output_sz, get_rand_shift()) for sigma in augs['blur']])
        if 'scale' in augs:
            self.transforms.extend(
                [augmentation.Scale(scale_factor, aug_output_sz, get_rand_shift()) for scale_factor in augs['scale']])
        if 'rotate' in augs:
            self.transforms.extend(
                [augmentation.Rotate(angle, aug_output_sz, get_rand_shift()) for angle in augs['rotate']])

        # Extract augmented image patches
        im_patches = sample_patch_transformed(im, self.init_sample_pos, self.init_sample_scale, aug_expansion_sz,
                                              self.transforms)

        # Extract initial backbone features
        with torch.no_grad():
            init_backbone_feat_iou, init_backbone_feat_secondF = self.net.extract_backbone(im_patches)

        return init_backbone_feat_iou, init_backbone_feat_secondF

    def init_target_boxes(self):
        """Get the target bounding boxes for the initial augmented samples."""
        self.classifier_target_box = self.get_iounet_box(self.pos, self.target_sz, self.init_sample_pos,
                                                         self.init_sample_scale)
        init_target_boxes = TensorList()
        for T in self.transforms:
            init_target_boxes.append(self.classifier_target_box + torch.Tensor([T.shift[1], T.shift[0], 0, 0]))
        init_target_boxes = torch.cat(init_target_boxes.view(1, 4), 0).to(self.params.device)
        self.target_boxes = init_target_boxes.new_zeros(self.params.sample_memory_size, 4)
        self.target_boxes[:init_target_boxes.shape[0], :] = init_target_boxes
        return init_target_boxes

    def init_memory(self, train_x1: TensorList, train_x2: TensorList):  # , train_x2: TensorList):
        # Initialize first-frame spatial training samples
        self.num_init_samples = train_x1.size(0)
        init_sample_weights = TensorList([x.new_ones(1) / x.shape[0] for x in train_x1])

        # Sample counters and weights for spatial
        self.num_stored_samples = self.num_init_samples.copy()
        self.previous_replace_ind = [None] * len(self.num_stored_samples)
        self.sample_weights = TensorList([x.new_zeros(self.params.sample_memory_size) for x in train_x1])
        for sw, init_sw, num in zip(self.sample_weights, init_sample_weights, self.num_init_samples):
            sw[:num] = init_sw

        # Initialize memory
        self.training_samples1 = TensorList(
            [x.new_zeros(self.params.sample_memory_size, x.shape[1], x.shape[2], x.shape[3]) for x in train_x1])

        self.training_samples2 = TensorList(
            [x.new_zeros(self.params.sample_memory_size, x.shape[1], x.shape[2], x.shape[3]) for x in train_x2])

        for ts, x in zip(self.training_samples1, train_x1):
            ts[:x.shape[0], ...] = x

        for ts, x in zip(self.training_samples2, train_x2):
            ts[:x.shape[0],...] = x

    def update_memory(self, sample_x1: TensorList, sample_x2: TensorList, target_box, learning_rate=None):  # sample_x2: TensorList,
        # Update weights and get replace ind
        replace_ind = self.update_sample_weights(self.sample_weights, self.previous_replace_ind,
                                                 self.num_stored_samples, self.num_init_samples, learning_rate)
        self.previous_replace_ind = replace_ind

        # Update sample and label memory
        for train_samp, x, ind in zip(self.training_samples1, sample_x1, replace_ind):
            train_samp[ind:ind + 1, ...] = x
        for train_samp, x, ind in zip(self.training_samples2, sample_x2, replace_ind):
            train_samp[ind:ind+1,...] = x

        # Update bb memory
        self.target_boxes[replace_ind[0], :] = target_box

        self.num_stored_samples += 1

    def update_sample_weights(self, sample_weights, previous_replace_ind, num_stored_samples, num_init_samples,
                              learning_rate=None):
        # Update weights and get index to replace
        replace_ind = []
        for sw, prev_ind, num_samp, num_init in zip(sample_weights, previous_replace_ind, num_stored_samples,
                                                    num_init_samples):
            lr = learning_rate
            if lr is None:
                lr = self.params.learning_rate

            init_samp_weight = getattr(self.params, 'init_samples_minimum_weight', None)
            if init_samp_weight == 0:
                init_samp_weight = None
            s_ind = 0 if init_samp_weight is None else num_init

            if num_samp == 0 or lr == 1:
                sw[:] = 0
                sw[0] = 1
                r_ind = 0
            else:
                # Get index to replace
                if num_samp < sw.shape[0]:
                    r_ind = num_samp
                else:
                    _, r_ind = torch.min(sw[s_ind:], 0)
                    r_ind = r_ind.item() + s_ind

                # Update weights
                if prev_ind is None:
                    sw /= 1 - lr
                    sw[r_ind] = lr
                else:
                    sw[r_ind] = sw[prev_ind] / (1 - lr)

            sw /= sw.sum()
            if init_samp_weight is not None and sw[:num_init].sum() < init_samp_weight:
                sw /= init_samp_weight + sw[num_init:].sum()
                sw[:num_init] = init_samp_weight / num_init

            replace_ind.append(r_ind)

        return replace_ind

    def update_state(self, new_pos, new_scale=None):
        # Update scale
        if new_scale is not None:
            self.target_scale = new_scale.clamp(self.min_scale_factor, self.max_scale_factor)
            self.target_sz = self.base_target_sz * self.target_scale

        # Update pos
        inside_ratio = getattr(self.params, 'target_inside_ratio', 0.2)
        inside_offset = (inside_ratio - 0.5) * self.target_sz
        self.pos = torch.max(torch.min(new_pos, self.image_sz - inside_offset), inside_offset)

    def get_iounet_box(self, pos, sz, sample_pos, sample_scale):
        """All inputs in original image coordinates.
        Generates a box in the cropped image sample reference frame, in the format used by the IoUNet."""
        box_center = (pos - sample_pos) / sample_scale + (self.img_sample_sz - 1) / 2
        box_sz = sz / sample_scale
        target_ul = box_center - (box_sz - 1) / 2
        return torch.cat([target_ul.flip((0,)), box_sz.flip((0,))])

    def init_iou_net(self, backbone_feat):
        # Setup IoU net and objective
        for p in self.net.dimp_pretrained.bb_regressor.parameters():
            p.requires_grad = False

        # Get target boxes for the different augmentations
        self.classifier_target_box = self.get_iounet_box(self.pos, self.target_sz, self.init_sample_pos,
                                                         self.init_sample_scale)
        target_boxes = TensorList()
        if self.params.iounet_augmentation:
            for T in self.transforms:
                if not isinstance(T, (
                augmentation.Identity, augmentation.Translation, augmentation.FlipHorizontal, augmentation.FlipVertical,
                augmentation.Blur)):
                    break
                target_boxes.append(self.classifier_target_box + torch.Tensor([T.shift[1], T.shift[0], 0, 0]))
        else:
            target_boxes.append(self.classifier_target_box + torch.Tensor(
                [self.transforms[0].shift[1], self.transforms[0].shift[0], 0, 0]))
        target_boxes = torch.cat(target_boxes.view(1, 4), 0).to(self.params.device)

        # Get iou features
        iou_backbone_feat = self.get_iou_backbone_features(backbone_feat)

        # Remove other augmentations such as rotation
        iou_backbone_feat = TensorList([x[:target_boxes.shape[0], ...] for x in iou_backbone_feat])

        # Get modulation vector
        self.iou_modulation = self.get_iou_modulation(iou_backbone_feat, target_boxes)
        self.iou_modulation = TensorList([x.detach().mean(0) for x in self.iou_modulation])

    def init_classifier(self, init_backbone_feat_iou, init_backbone_feat_secondF):
        # Get classification features  x2,
        x1, x2 = self.get_classification_features(init_backbone_feat_iou, init_backbone_feat_secondF )
        """
        init_backbone_feat = dict{2}: layer2:[23, 512, 28, 28] layer3:[23, 1024, 14, 14]
        x = tensor[23, 512, 14, 14].

        x2 = tensor[23, 512, 14, 14].
        x1 = tensor[23, 512, 14, 14].
        """
        # Add the dropout augmentation here, since it requires extraction of the classification features
        if 'dropout' in self.params.augmentation and getattr(self.params, 'use_augmentation', True):
            num, prob = self.params.augmentation['dropout']
            self.transforms.extend(self.transforms[:1] * num)
            x1 = torch.cat([x1, F.dropout2d(x1[0:1, ...].expand(num, -1, -1, -1), p=prob, training=True)])
            x2 = torch.cat([x2, F.dropout2d(x2[0:1,...].expand(num,-1,-1,-1), p=prob, training=True)])

        # Set feature size and other related sizes
        self.feature_sz = torch.Tensor(list(x1.shape[-2:]))
        # self.feature_sz2 = torch.Tensor(list(x2.shape[-2:]))

        ksz = self.net.classifier.filter_size
        ksz2 = self.net.dimp_pretrained.classifier.filter_size

        self.kernel_size = torch.Tensor([ksz, ksz] if isinstance(ksz, (int, float)) else ksz)
        # self.kernel_size2= torch.Tensor([ksz2, ksz2] if isinstance(ksz2, (int, float)) else ksz2)

        self.output_sz = self.feature_sz + (self.kernel_size + 1) % 2
        # self.output_sz2 = self.feature_sz2 + (self.kernel_size2 + 1)%2

        # Construct output window
        self.output_window = None
        if getattr(self.params, 'window_output', False):
            if getattr(self.params, 'use_clipped_window', False):
                self.output_window = dcf.hann2d_clipped(self.output_sz.long(),
                                                        self.output_sz.long() * self.params.effective_search_area / self.params.search_area_scale,
                                                        centered=False).to(self.params.device)
                # self.output_window2 = dcf.hann2d_clipped(self.output_sz2.long(), self.output_sz2.long()*self.params.effective_search_area / self.params.search_area_scale, centered=False).to(self.params.device)

            else:
                self.output_window = dcf.hann2d(self.output_sz.long(), centered=True).to(self.params.device)
                # self.output_window2 = dcf.hann2d(self.output_sz2.long(), centered=True).to(self.params.device)

            self.output_window = self.output_window.squeeze(0)
            # self.output_window2 = self.output_window2.squeeze(0)

        # Get target boxes for the different augmentations
        target_boxes = self.init_target_boxes()

        # Set number of iterations
        plot_loss = self.params.debug > 0
        num_iter = getattr(self.params, 'net_opt_iter', None)

        # Get target filter by running the discriminative model prediction module
        with torch.no_grad():
            self.target_filter1, _, losses = self.net.classifier.get_filter(x1, target_boxes, num_iter=num_iter,
                                                                           compute_losses=plot_loss)
            self.target_filter2, _, losses = self.net.dimp_pretrained.classifier.get_filter(x2, target_boxes, num_iter=num_iter,
                                                                           compute_losses=plot_loss)

        # Init memory
        if getattr(self.params, 'update_classifier', True):
            self.init_memory(TensorList([x1]), TensorList([x2]))  # , TensorList([x2])

        if plot_loss:
            if isinstance(losses, dict):
                losses = losses['train']
            self.losses = torch.cat(losses)
            if self.visdom is not None:
                self.visdom.register((self.losses, torch.arange(self.losses.numel())), 'lineplot', 3, 'Training Loss')
            elif self.params.debug >= 3:
                plot_graph(self.losses, 10, title='Training loss')

    def update_classifier(self, train_x1, train_x2, target_box, learning_rate=None, scores=None):  # train_x2,
        # Set flags and learning rate
        hard_negative_flag = learning_rate is not None
        if learning_rate is None:
            learning_rate = self.params.learning_rate

        # Update the tracker memory
        self.update_memory(TensorList([train_x1]), TensorList([train_x2]), target_box,
                           learning_rate)  # TensorList([train_x2]),

        # Decide the number of iterations to run
        num_iter = 0
        low_score_th = getattr(self.params, 'low_score_opt_threshold', None)
        if hard_negative_flag:
            num_iter = getattr(self.params, 'net_opt_hn_iter', None)
        elif low_score_th is not None and low_score_th > scores.max().item():
            num_iter = getattr(self.params, 'net_opt_low_iter', None)
        elif (self.frame_num - 1) % self.params.train_skipping == 0:
            num_iter = getattr(self.params, 'net_opt_update_iter', None)

        plot_loss = self.params.debug > 0

        if num_iter > 0:
            # Get inputs for the DiMP filter optimizer module
            samples1 = self.training_samples1[0][:self.num_stored_samples[0], ...]
            samples2 = self.training_samples2[0][:self.num_stored_samples[0],...]

            target_boxes = self.target_boxes[:self.num_stored_samples[0], :].clone()
            sample_weights = self.sample_weights[0][:self.num_stored_samples[0]]

            # Run the filter optimizer module
            with torch.no_grad():
                self.target_filter1, _, losses = self.net.classifier.filter_optimizer(self.target_filter1, samples1,
                                                                                     target_boxes,
                                                                                     sample_weight=sample_weights,
                                                                                     num_iter=num_iter,
                                                                                     compute_losses=plot_loss)
                self.target_filter2, _, losses = self.net.dimp_pretrained.classifier.filter_optimizer(self.target_filter2, samples2,
                                                                                       target_boxes,
                                                                                       sample_weight=sample_weights,
                                                                                       num_iter=num_iter,
                                                                                       compute_losses=plot_loss)

            if plot_loss:
                if isinstance(losses, dict):
                    losses = losses['train']
                self.losses = torch.cat((self.losses, torch.cat(losses)))
                if self.visdom is not None:
                    self.visdom.register((self.losses, torch.arange(self.losses.numel())), 'lineplot', 3,
                                         'Training Loss')
                elif self.params.debug >= 3:
                    plot_graph(self.losses, 10, title='Training loss')

    def refine_target_box(self, backbone_feat, sample_pos, sample_scale, scale_ind, update_scale=True):
        """Run the ATOM IoUNet to refine the target bounding box."""

        # Initial box for refinement
        init_box = self.get_iounet_box(self.pos, self.target_sz, sample_pos, sample_scale)

        # Extract features from the relevant scale
        iou_features = self.get_iou_features(backbone_feat)
        iou_features = TensorList([x[scale_ind:scale_ind + 1, ...] for x in iou_features])

        # Generate random initial boxes
        init_boxes = init_box.view(1, 4).clone()
        if self.params.num_init_random_boxes > 0:
            square_box_sz = init_box[2:].prod().sqrt()
            rand_factor = square_box_sz * torch.cat(
                [self.params.box_jitter_pos * torch.ones(2), self.params.box_jitter_sz * torch.ones(2)])

            minimal_edge_size = init_box[2:].min() / 3
            rand_bb = (torch.rand(self.params.num_init_random_boxes, 4) - 0.5) * rand_factor
            new_sz = (init_box[2:] + rand_bb[:, 2:]).clamp(minimal_edge_size)
            new_center = (init_box[:2] + init_box[2:] / 2) + rand_bb[:, :2]
            init_boxes = torch.cat([new_center - new_sz / 2, new_sz], 1)
            init_boxes = torch.cat([init_box.view(1, 4), init_boxes])

        # Optimize the boxes
        output_boxes, output_iou = self.optimize_boxes(iou_features, init_boxes)

        # Remove weird boxes
        output_boxes[:, 2:].clamp_(1)
        aspect_ratio = output_boxes[:, 2] / output_boxes[:, 3]
        keep_ind = (aspect_ratio < self.params.maximal_aspect_ratio) * (
                    aspect_ratio > 1 / self.params.maximal_aspect_ratio)
        output_boxes = output_boxes[keep_ind, :]
        output_iou = output_iou[keep_ind]

        # If no box found
        if output_boxes.shape[0] == 0:
            return

        # Predict box
        k = getattr(self.params, 'iounet_k', 5)
        topk = min(k, output_boxes.shape[0])
        _, inds = torch.topk(output_iou, topk)
        predicted_box = output_boxes[inds, :].mean(0)
        predicted_iou = output_iou.view(-1, 1)[inds, :].mean(0)

        # Get new position and size
        new_pos = predicted_box[:2] + predicted_box[2:] / 2
        new_pos = (new_pos.flip((0,)) - (self.img_sample_sz - 1) / 2) * sample_scale + sample_pos
        new_target_sz = predicted_box[2:].flip((0,)) * sample_scale
        new_scale = torch.sqrt(new_target_sz.prod() / self.base_target_sz.prod())

        self.pos_iounet = new_pos.clone()

        if getattr(self.params, 'use_iounet_pos_for_learning', True):
            self.pos = new_pos.clone()

        self.target_sz = new_target_sz

        if update_scale:
            self.target_scale = new_scale

    def optimize_boxes(self, iou_features, init_boxes):
        # Optimize iounet boxes
        output_boxes = init_boxes.view(1, -1, 4).to(self.params.device)
        step_length = self.params.box_refinement_step_length
        if isinstance(step_length, (tuple, list)):
            step_length = torch.Tensor([step_length[0], step_length[0], step_length[1], step_length[1]],
                                       device=self.params.device).view(1, 1, 4)

        for i_ in range(self.params.box_refinement_iter):
            # forward pass
            bb_init = output_boxes.clone().detach()
            bb_init.requires_grad = True

            outputs = self.net.dimp_pretrained.bb_regressor.predict_iou(self.iou_modulation, iou_features, bb_init)

            if isinstance(outputs, (list, tuple)):
                outputs = outputs[0]

            outputs.backward(gradient=torch.ones_like(outputs))

            # Update proposal
            output_boxes = bb_init + step_length * bb_init.grad * bb_init[:, :, 2:].repeat(1, 1, 2)
            output_boxes.detach_()

            step_length *= self.params.box_refinement_step_decay

        return output_boxes.view(-1, 4).cpu(), outputs.detach().view(-1).cpu()

    def to_device(self, device):
        self.device = device
