import votworkspace.vot2020a as vota
import sys
import importlib
from pytracking.evaluation.environment import env_settings
import cv2
import numpy as np
import time


try:
    import trax
except ImportError:
    raise Exception('TraX support not found. Please add trax module to Python path.')


def get_parameters(tracker_name, param_file_name):
    param_module = importlib.import_module('pytracking.parameter.{}.{}'.format(tracker_name, param_file_name))
    params = param_module.parameters()
    return params

def read_image(image_file: str):
        return cv2.cvtColor(cv2.imread(image_file), cv2.COLOR_BGR2RGB)

def rect_to_poly(rect):
    x0 = rect[0]
    y0 = rect[1]
    x1 = rect[0] + rect[2]
    y1 = rect[1]
    x2 = rect[0] + rect[2]
    y2 = rect[1] + rect[3]
    x3 = rect[0]
    y3 = rect[1] + rect[3]
    return [x0, y0, x1, y1, x2, y2, x3, y3]

def get_axis_aligned_bbox(region):
    nv = region.size
    if nv == 8:
        cx = np.mean(region[0::2])
        cy = np.mean(region[1::2])
        x1 = min(region[0::2])
        x2 = max(region[0::2])
        y1 = min(region[1::2])
        y2 = max(region[1::2])
        A1 = np.linalg.norm(region[0:2] -
                            region[2:4]) * np.linalg.norm(region[2:4] -
                                                          region[4:6])
        A2 = (x2 - x1) * (y2 - y1)
        s = np.sqrt(A1 / A2)
        w = s * (x2 - x1) + 1
        h = s * (y2 - y1) + 1
    else:
        x = region[0]
        y = region[1]
        w = region[2]
        h = region[3]
        cx = x + (w - 1) / 2
        cy = y + (h - 1) / 2

    return cx, cy, w, h

def cxy_wh_2_rect(pos, sz):
    return np.array([pos[0] - sz[0] / 2, pos[1] - sz[1] / 2, sz[0],
                     sz[1]])  # 0-index

def parse_sequence_name(image_path):
    idx = image_path.find('/color/')
    return image_path[idx - image_path[:idx][::-1].find('/'):idx], idx

def parse_frame_name(image_path, idx):
    frame_name = image_path[idx + len('/color/'):]
    return frame_name[:frame_name.find('.')]

tracker_module_dimp50 = importlib.import_module('pytracking.tracker.{}'.format('dimp'))
tracker_class_dimp50 = tracker_module_dimp50.get_tracker_class()
params_dimp50_vot20 = get_parameters('dimp', 'dimp50_vot19')
dimp50_vot20 = tracker_class_dimp50(params_dimp50_vot20)

tracker = dimp50_vot20

input_type = 'poly'

if input_type =='poly':
     #a = trax.Region.POLYGON # a = 'polygon'

     handle = vota.VOT('polygon')  # handle = vot.VOT("polygon")
     selection = handle.region()
     imagefile = handle.frame()
     if not imagefile:
        sys.exit(0)

     gt_rect = [round(selection.points[0].x, 2), round(selection.points[0].y, 2),
                round(selection.points[1].x, 2), round(selection.points[1].y, 2),
                round(selection.points[2].x, 2), round(selection.points[2].y, 2),
                round(selection.points[3].x, 2), round(selection.points[3].y, 2)]

     image = cv2.cvtColor(cv2.imread(imagefile), cv2.COLOR_BGR2RGB)

     tracker.init(image, gt_rect)

     while True:
         imagefile = handle.frame()
         if not imagefile:
            break

         image = cv2.cvtColor(cv2.imread(imagefile), cv2.COLOR_BGR2RGB)

         prediction = tracker.update(image)


         pred_poly = vota.Polygon([vota.Point(prediction[0], prediction[1]),
                                  vota.Point(prediction[2], prediction[3]),
                                  vota.Point(prediction[4], prediction[5]),
                                  vota.Point(prediction[6], prediction[7])])

         handle.report(pred_poly)
