from __future__ import absolute_import

import numpy as np
import time
from PIL import Image

from ..utils.viz import show_frame
import os
import cv2

class Tracker(object):

    def __init__(self, name, is_deterministic=False):
        self.name = name
        self.is_deterministic = is_deterministic
    
    def init(self, image, box):
        raise NotImplementedError()

    def update(self, image):
        raise NotImplementedError()

    def track(self, img_files, box, visualize=False):
        frame_num = len(img_files)
        boxes = np.zeros((frame_num, 4))
        boxes[0] = box[0, :]
        times = np.zeros(frame_num)

        for f, img_file in enumerate(img_files):
            image = Image.open(img_file)
            if not image.mode == 'RGB':
                image = image.convert('RGB')

  #          image = cv2.cvtColor(cv2.imread(img_file), cv2.COLOR_BGR2RGB)
            start_time = time.time()
            if f == 0:
                self.init(image, box[0, :])
            else:
                track_rs = self.update(image)

                boxes[f, 0] = track_rs[0]
                boxes[f, 1] = track_rs[1]
                boxes[f, 2] = track_rs[2] - track_rs[0]
                boxes[f, 3] = track_rs[3] - track_rs[1]

            times[f] = time.time() - start_time


            if visualize == True :

                img_show = cv2.imread(img_file)
                # bbox xmin ymin xmax ymax
                bbox = boxes[f, :]
                img_show = cv2.rectangle(img_show,
                              (int(bbox[0]), int(bbox[1])),
                              (int(bbox[0] + bbox[2]), int(bbox[1] + bbox[3])),
                              (0, 0, 255),
                              1)

                gt_bbox = box[f, :]
                gt_bbox = (gt_bbox[0], gt_bbox[1],
                               gt_bbox[0] + gt_bbox[2], gt_bbox[1] + gt_bbox[3])
                img_show = cv2.rectangle(img_show,
                                  (int(gt_bbox[0] - 1), int(gt_bbox[1] - 1)),  # 0-index
                                  (int(gt_bbox[2] - 1), int(gt_bbox[3] - 1)),
                                   (0, 255, 0), 2)
              #  GT is red, siamfc is green
                img_show = cv2.putText(img_show, str(f), (5, 20), cv2.FONT_HERSHEY_COMPLEX_SMALL, 1, (0, 255, 0), 1)
                title = 'testing'

                cv2.imshow(title, img_show)
                cv2.waitKey(5)

        return boxes, times


from .identity_tracker import IdentityTracker
